define([
  "dojo/_base/declare",
  "dojo/dom-style",
  "dojo/keys",
  "dojo/on",
  "dijit/form/FilteringSelect"
], function(declare, domStyle, keys, on, FilteringSelect) {
  return declare("dijit.form.FilteringSelectEx", FilteringSelect, {
    enterKeyAction: "ignore", // Enter キー設定
    imeMode: "auto",          // IME モード設定
    cursorPosition: "left",   // left|right "left"
    searchDelay: 300,
    identity: null,
    content: null,

    postCreate: function() {
      this.inherited('postCreate', arguments);
      if (this.imeMode !== "" && this.imeMode !== "auto") {
        domStyle.set(this.textbox, "imeMode", this.imeMode);
      }
      this.setCursorPosition();
      this.setItem();
      this.setEnterKeyAction();
    },

    setCursorPosition: function() {
      if (this.cursorPosition === "left") {
        return;
      }
      domStyle.set(this.textbox, "textAlign", this.cursorPosition);
    },

    setEnterKeyAction: function() {
      if (!this.enterKeyAction || this.enterKeyAction === "") {
        return;
      }
      var that = this;
      on(this.textbox, "keydown", function(evt) {
        that._onKeyDown(evt);
      });
    },

    _onKeyDown: function(/*Event*/ evt) {
      if (evt.keyCode !== keys.ENTER) {
        return;
      }
      if (this.enterKeyAction === "changetab" ||
          this.enterKeyAction === "ignore") {
        evt.preventDefault();
      }
      if (this.enterKeyAction === "changetab") {
        enterKeyHandler.focusNextElement(this.textbox, evt);
      } else if (this.enterKeyAction === "ignore") {
        var pw = this.dropDown;
        if(pw && pw.getHighlightedOption()) {
          this._selectOption();
          this.closeDropDown();
        }
      } else if (this.enterKeyAction === "default") {
        this.onChange();
        doSubmitForm(evt.srcElement.form, 'action_Send');
      }
    },

    setItem: function() {
      if (!(this.store && this.identity && this.content)) {
        return;
      }
      var property = this.getPropertyNameFromStoreURL();
      if (!property) {
        return;
      }
      var i = {};
      i[property.targetname] = this.content;
      i[property.idname] = this.identity;
      this.set("item", {
        'i': i,
        'r': this.store,
      });
    },

    getPropertyNameFromStoreURL: function() {
      if (!this.store.url) {
        return null;
      }
      var args = this.store.url.split("&");
      if (!args || args.length === 0) {
        return null;
      }
      var property = {};
      for (var i=0,l=args.length; i<l; i++) {
        if (!(args[i].lastIndexOf("targetname=", 0) === 0 || args[i].lastIndexOf("idname=", 0) === 0)) {
          continue;
        }
        var props = args[i].split("=");
        property[props[0]] = props[1];
      }
      return property;
    },

    _callbackSetLabel: function(result, query, options, priorityChange){
      if (!options) {
        // 1.2で新たに加わった条件式(下記関数内最初のif文)を回避する
        // これがないと更新画面表示時に値がセットされなくなる
        if (result && result.length > 0) {
          this._lastQuery = this.store.getIdentity(result[0]);
        }
      }
      this.inherited('_callbackSetLabel', arguments);
    },

    _startSearch: function(text) {
      if (!text) {
        text = "*";
      }
      this.inherited('_startSearch', arguments);
    },

    _setDisplayedValueAttr: function(label, priorityChange) {
      if (!label) {
        return;
      }
      this.inherited('_setDisplayedValueAttr', arguments);
    },

  });
});
